# -*- coding: utf-8 -*-
#
# Copyright © 2012 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
rotatecrop
----------

The `rotatecrop` module provides a dialog box providing essential GUI elements 
for rotating (arbitrary angle) and cropping an image:
    
    * :py:class:`guiqwt.widgets.rotatecrop.RotateCropDialog`: dialog box
    * :py:class:`guiqwt.widgets.rotatecrop.RotateCropWidget`: equivalent widget

Reference
~~~~~~~~~

.. autoclass:: RotateCropDialog
   :members:
   :inherited-members:
.. autoclass:: RotateCropWidget
   :members:
   :inherited-members:
"""

from qtpy.QtWidgets import QCheckBox

# Local imports
from guiqwt.config import _
from guiqwt.builder import make
from guiqwt.image import get_image_in_shape
from guiqwt.widgets import base


class RotateCropMixin(base.BaseTransformMixin):
    """Rotate & Crop mixin class, to be mixed with a class providing the
    get_plot method, like ImageDialog or RotateCropWidget (see below)"""

    def __init__(self):
        base.BaseTransformMixin.__init__(self)
        self.crop_rect = None

    # ------BaseTransformMixin API----------------------------------------------
    def add_buttons_to_layout(self, layout):
        """Add tool buttons to layout"""
        # Show crop rectangle checkbox
        show_crop = QCheckBox(_("Show cropping rectangle"), self)
        show_crop.setChecked(True)
        show_crop.toggled.connect(self.show_crop_rect)
        layout.addWidget(show_crop)
        layout.addSpacing(15)
        base.BaseTransformMixin.add_buttons_to_layout(self, layout)

    def set_item(self, item):
        """Set associated item -- must be a TrImageItem object"""
        base.BaseTransformMixin.set_item(self, item)
        crect = make.annotated_rectangle(0, 0, 1, 1, _("Cropping rectangle"))
        self.crop_rect = crect
        crect.annotationparam.format = "%.1f cm"
        plot = self.get_plot()
        plot.add_item(crect)
        plot.set_active_item(crect)
        x0, y0, x1, y1 = self.item.get_crop_coordinates()
        crect.set_rect(x0, y0, x1, y1)
        plot.replot()

    def reset_transformation(self):
        """Reset transformation"""
        x0, y0, x1, y1 = self.item.border_rect.get_rect()
        self.crop_rect.set_rect(x0, y0, x1, y1)

    def apply_transformation(self):
        """Apply transformation, e.g. crop or rotate"""
        # Let's crop!
        i_points = self.item.border_rect.get_points()
        xmin, ymin = i_points.min(axis=0)
        xmax, ymax = i_points.max(axis=0)
        xc0, yc0, xc1, yc1 = self.crop_rect.shape.get_rect()
        left = max([0, xc0 - xmin])
        right = max([0, xmax - xc1])
        top = max([0, yc0 - ymin])
        bottom = max([0, ymax - yc1])
        self.item.set_crop(left, top, right, bottom)
        #        print "set_crop:", left, top, right, bottom
        self.item.compute_bounds()
        self.get_plot().replot()

    def compute_transformation(self):
        """Compute transformation, return compute output array"""
        return get_image_in_shape(self.crop_rect, apply_interpolation=False)

    # ------Private API---------------------------------------------------------
    def show_crop_rect(self, state):
        """Show/hide cropping rectangle shape"""
        self.crop_rect.setVisible(state)
        self.crop_rect.label.setVisible(state)
        self.get_plot().replot()


class RotateCropDialog(base.BaseTransformDialog, RotateCropMixin):
    """Rotate & Crop Dialog

    Rotate and crop a :py:class:`guiqwt.image.TrImageItem` plot item"""

    def __init__(self, parent, wintitle=None, options=None, resize_to=None):
        RotateCropMixin.__init__(self)
        base.BaseTransformDialog.__init__(
            self, parent, wintitle=wintitle, options=options, resize_to=resize_to
        )


class RotateCropWidget(base.BaseTransformWidget, RotateCropMixin):
    """Rotate & Crop Widget

    Rotate and crop a :py:class:`guiqwt.image.TrImageItem` plot item"""

    def __init__(self, parent, options=None):
        base.BaseTransformWidget.__init__(self, parent, options=options)
        RotateCropMixin.__init__(self)


class MultipleRotateCropWidget(base.BaseMultipleTransformWidget):
    """Multiple Rotate & Crop Widget

    Rotate and crop several :py:class:`guiqwt.image.TrImageItem` plot items"""

    TRANSFORM_WIDGET_CLASS = RotateCropWidget



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

# pylint: disable-msg=W0622,R0903
# W0622: complains about filter being a builtin
# R0903: complains about too few public methods which is the purpose here

"""
guiqwt.events
-------------

The `event` module handles event management (states, event filter, ...).
"""


import weakref
from qtpy.QtCore import QEvent, Qt, QObject, QPointF, Signal
from qtpy.QtGui import QKeySequence

CursorShape = type(Qt.ArrowCursor)

from guiqwt.config import CONF
from guiqwt.debug import evt_type_to_str, buttons_to_str


# Sélection d'événements  ---------
class EventMatch(object):
    """A callable returning true if it matches an event"""

    def __call__(self, event):
        raise NotImplementedError

    def get_event_types(self):
        """Returns a set of event types handled by this
        EventMatch.
        This is used to quickly optimize events not handled
        by any event matchers
        """
        return frozenset()


class KeyEventMatch(EventMatch):
    """
    A callable returning True if it matches a key event
    keys: list of keys or couples (key, modifier)
    """

    def __init__(self, keys):
        super(KeyEventMatch, self).__init__()
        key_list, mod_list = [], []
        for item in keys:
            if isinstance(item, (tuple, list)):
                k, m = item
            else:
                k = item
                # Avoid bad arguments: modifier instead of key
                assert k not in (
                    Qt.ControlModifier,
                    Qt.ShiftModifier,
                    Qt.AltModifier,
                    Qt.NoModifier,
                )
                m = Qt.NoModifier
            key_list.append(k)
            mod_list.append(m)
        self.keys = key_list
        self.mods = mod_list

    def get_event_types(self):
        return frozenset((QEvent.KeyPress,))

    def __call__(self, event):
        if event.type() == QEvent.KeyPress:
            my_key = event.key()
            my_mod = event.modifiers()
            if my_key in self.keys:
                mod = self.mods[self.keys.index(my_key)]
                if mod == Qt.NoModifier or my_mod & mod:
                    return True
        return False


class StandardKeyMatch(EventMatch):
    """
    A callable returning True if it matches a key event
    keysequence: QKeySequence.StandardKey integer
    """

    def __init__(self, keysequence):
        super(StandardKeyMatch, self).__init__()
        assert isinstance(keysequence, (int, QKeySequence.StandardKey))
        self.keyseq = keysequence

    def get_event_types(self):
        return frozenset((QEvent.KeyPress,))

    def __call__(self, event):
        return event.type() == QEvent.KeyPress and event.matches(self.keyseq)


class MouseEventMatch(EventMatch):
    """Base class for matching mouse events"""

    def __init__(self, evt_type, btn, modifiers=Qt.NoModifier):
        super(MouseEventMatch, self).__init__()
        assert isinstance(modifiers, (int, Qt.KeyboardModifiers, Qt.KeyboardModifier))
        self.evt_type = evt_type
        self.button = btn
        self.modifiers = modifiers

    def get_event_types(self):
        return frozenset((self.evt_type,))

    def __call__(self, event):
        if event.type() == self.evt_type:
            if event.button() == self.button:
                if self.modifiers != Qt.NoModifier:
                    if (event.modifiers() & self.modifiers) == self.modifiers:
                        return True
                elif event.modifiers() == Qt.NoModifier:
                    return True
        return False

    def __repr__(self):
        return "<MouseMatch: %s/ %08x:%s>" % (
            evt_type_to_str(self.evt_type),
            self.modifiers,
            buttons_to_str(self.button),
        )


class MouseMoveMatch(MouseEventMatch):
    def __call__(self, event):
        if event.type() == self.evt_type:
            if (
                self.button != Qt.NoButton
                and (event.buttons() & self.button == self.button)
            ) or event.buttons() == Qt.NoButton:
                if self.modifiers != Qt.NoModifier:
                    if (event.modifiers() & self.modifiers) == self.modifiers:
                        return True
                elif event.modifiers() == Qt.NoModifier:
                    return True
        return False


# Machine d'état  ----------
class StatefulEventFilter(QObject):
    """Gestion d'une machine d'état pour les événements
    d'un canvas
    """

    def __init__(self, parent):
        super(StatefulEventFilter, self).__init__()
        self.states = {0: {}}  # 0 : cursor 1: panning, 2: zooming
        self.cursors = {}
        self.state = 0
        self.max_state = 0
        self.events = {}
        self.plot = parent
        self.all_event_types = frozenset()

    def eventFilter(self, _obj, event):
        """Le callback 'eventfilter' pour Qt"""
        if not hasattr(self, "all_event_types"):
            print(repr(self), self)
        if event.type() not in self.all_event_types:
            return False

        state = self.states[self.state]
        #        from pprint import pprint
        #        print self.state
        #        pprint(state.keys())
        for match, (call_list, next_state) in list(state.items()):
            if match(event):
                self.set_state(next_state, event)
                for call in call_list:
                    call(self, event)  # might change state
        return False

    def set_state(self, state, event):
        """Change l'état courant.

        Peut être appelé par les handlers pour annuler un
        changement d'état"""
        assert state in self.states
        if state == self.state:
            return
        self.state = state
        cursor = self.get_cursor(event)
        if cursor is not None:
            self.plot.canvas().setCursor(cursor)

    def new_state(self):
        """Création (réservation) d'un nouveau numéro d'état"""
        self.max_state += 1
        self.states[self.max_state] = {}
        return self.max_state

    def add_event(self, state, match, call, next_state=None):
        """Ajoute une transition sur la machine d'état
        si next_state est fourni, il doit correspondre à un état existant
        sinon un nouvel état d'arrivée est créé
        """
        assert isinstance(state, int)
        assert isinstance(match, EventMatch)
        self.all_event_types = self.all_event_types.union(match.get_event_types())
        entry = self.states[state].setdefault(match, [[], None])
        entry[0].append(call)
        if entry[1] is None:
            if next_state is None:
                next_state = self.new_state()
            else:
                pass
        else:
            if next_state is not None:
                assert next_state == entry[1]
            else:
                next_state = entry[1]

        entry[1] = next_state
        return next_state

    # gestion du curseur
    def set_cursor(self, cursor, *states):
        """Associe un curseur à un ou plusieurs états"""
        assert isinstance(cursor, CursorShape)
        for s in states:
            self.cursors[s] = cursor

    def get_cursor(self, _event):
        """Récupère le curseur associé à un état / événement donné"""
        # on passe event pour eventuellement pouvoir choisir
        # le curseur en fonction des touches de modification
        cursor = self.cursors.get(self.state, None)
        if cursor is None:
            # no cursor specified : should keep previous one
            return None
        return cursor

    # Fonction utilitaires
    def mouse_press(self, btn, modifiers=Qt.NoModifier):
        """Création d'un filtre pour l'événement MousePress"""
        return self.events.setdefault(
            ("mousepress", btn, modifiers),
            MouseEventMatch(QEvent.MouseButtonPress, btn, modifiers),
        )

    def mouse_move(self, btn, modifiers=Qt.NoModifier):
        """Création d'un filtre pour l'événement MouseMove"""
        return self.events.setdefault(
            ("mousemove", btn, modifiers),
            MouseMoveMatch(QEvent.MouseMove, btn, modifiers),
        )

    def mouse_release(self, btn, modifiers=Qt.NoModifier):
        """Création d'un filtre pour l'événement MouseRelease"""
        return self.events.setdefault(
            ("mouserelease", btn, modifiers),
            MouseEventMatch(QEvent.MouseButtonRelease, btn, modifiers),
        )

    def nothing(self, filter, event):
        """A nothing filter, provided to help removing duplicate handlers"""
        pass


class DragHandler(QObject):
    """Classe de base pour les gestionnaires d'événements du type
    click - drag - release
    """

    cursor = None

    def __init__(self, filter, btn, mods=Qt.NoModifier, start_state=0):
        super(DragHandler, self).__init__()
        self.state0 = filter.add_event(
            start_state, filter.mouse_press(btn, mods), self.start_tracking
        )
        self.state1 = filter.add_event(
            self.state0, filter.mouse_move(btn, mods), self.start_moving
        )
        filter.add_event(
            self.state1, filter.mouse_move(btn, mods), self.move, self.state1
        )
        filter.add_event(
            self.state0,
            filter.mouse_release(btn, mods),
            self.stop_notmoving,
            start_state,
        )
        filter.add_event(
            self.state1, filter.mouse_release(btn, mods), self.stop_moving, start_state
        )
        if self.cursor is not None:
            filter.set_cursor(self.cursor, self.state0, self.state1)
        self.start = None  # first mouse position
        self.last = None  # mouse position seen during last event
        self.parent_tracking = None

    def get_move_state(self, filter, pos):
        rct = filter.plot.contentsRect()
        dx = (pos.x(), self.last.x(), self.start.x(), rct.width())
        dy = (pos.y(), self.last.y(), self.start.y(), rct.height())
        self.last = QPointF(pos)
        return dx, dy

    def start_tracking(self, _filter, event):
        self.start = self.last = QPointF(event.pos())

    def start_moving(self, filter, event):
        return self.move(filter, event)

    def stop_tracking(self, _filter, _event):
        pass
        # filter.plot.canvas().setMouseTracking(self.parent_tracking)

    def stop_notmoving(self, filter, event):
        self.stop_tracking(filter, event)

    def stop_moving(self, filter, event):
        self.stop_tracking(filter, event)

    def move(self, filter, event):
        raise NotImplementedError()


class ClickHandler(QObject):
    """Classe de base pour les gestionnaires d'événements du type
    click - release
    """

    #: Signal emitted by ClickHandler on mouse click
    SIG_CLICK_EVENT = Signal(object, "QEvent")

    def __init__(self, filter, btn, mods=Qt.NoModifier, start_state=0):
        super(ClickHandler, self).__init__()
        self.state0 = filter.add_event(
            start_state, filter.mouse_press(btn, mods), filter.nothing
        )
        filter.add_event(
            self.state0, filter.mouse_release(btn, mods), self.click, start_state
        )

    def click(self, filter, event):
        self.SIG_CLICK_EVENT.emit(filter, event)


class PanHandler(DragHandler):
    cursor = Qt.ClosedHandCursor

    def move(self, filter, event):
        x_state, y_state = self.get_move_state(filter, event.pos())
        filter.plot.do_pan_view(x_state, y_state)


class ZoomHandler(DragHandler):
    cursor = Qt.SizeAllCursor

    def move(self, filter, event):
        x_state, y_state = self.get_move_state(filter, event.pos())
        filter.plot.do_zoom_view(x_state, y_state)


class MenuHandler(ClickHandler):
    def click(self, filter, event):
        menu = filter.plot.get_context_menu()
        if menu:
            menu.popup(event.globalPos())


class QtDragHandler(DragHandler):

    #: Signal emitted by QtDragHandler when starting tracking
    SIG_START_TRACKING = Signal(object, "QEvent")

    #: Signal emitted by QtDragHandler when stopping tracking and not moving
    SIG_STOP_NOT_MOVING = Signal(object, "QEvent")

    #: Signal emitted by QtDragHandler when stopping tracking and moving
    SIG_STOP_MOVING = Signal(object, "QEvent")

    #: Signal emitted by QtDragHandler when moving
    SIG_MOVE = Signal(object, "QEvent")

    def start_tracking(self, filter, event):
        DragHandler.start_tracking(self, filter, event)
        self.SIG_START_TRACKING.emit(filter, event)

    def stop_notmoving(self, filter, event):
        self.SIG_STOP_NOT_MOVING.emit(filter, event)

    def stop_moving(self, filter, event):
        self.SIG_STOP_MOVING.emit(filter, event)

    def move(self, filter, event):
        self.SIG_MOVE.emit(filter, event)


class AutoZoomHandler(ClickHandler):
    def click(self, filter, _event):
        filter.plot.do_autoscale()


class MoveHandler(object):
    def __init__(self, filter, btn=Qt.NoButton, mods=Qt.NoModifier, start_state=0):
        filter.add_event(
            start_state, filter.mouse_move(btn, mods), self.move, start_state
        )

    def move(self, filter, event):
        filter.plot.do_move_marker(event)


class UndoMoveObject(object):
    def __init__(self, obj, pos1, pos2):
        self.obj = obj
        from guiqwt.baseplot import canvas_to_axes

        self.coords1 = canvas_to_axes(obj, pos1)
        self.coords2 = canvas_to_axes(obj, pos2)

    def is_valid(self):
        return self.obj.plot() is not None

    def compute_positions(self):
        from guiqwt.baseplot import axes_to_canvas

        pos1 = QPointF(*axes_to_canvas(self.obj, *self.coords1))
        pos2 = QPointF(*axes_to_canvas(self.obj, *self.coords2))
        return pos1, pos2

    def undo(self):
        pos1, pos2 = self.compute_positions()
        self.obj.plot().unselect_all()
        self.obj.move_local_shape(pos1, pos2)

    def redo(self):
        pos1, pos2 = self.compute_positions()
        self.obj.plot().unselect_all()
        self.obj.move_local_shape(pos2, pos1)


class UndoMovePoint(UndoMoveObject):
    def __init__(self, obj, pos1, pos2, handle, ctrl):
        super(UndoMovePoint, self).__init__(obj, pos1, pos2)
        self.handle = handle
        self.ctrl = ctrl

    def undo(self):
        pos1, pos2 = self.compute_positions()
        self.obj.move_local_point_to(self.handle, pos1, self.ctrl)

    def redo(self):
        pos1, pos2 = self.compute_positions()
        self.obj.move_local_point_to(self.handle, pos2, self.ctrl)


class ObjectHandler(object):
    def __init__(
        self, filter, btn, mods=Qt.NoModifier, start_state=0, multiselection=False
    ):
        self.multiselection = multiselection
        self.start_state = start_state
        self.state0 = filter.add_event(
            start_state, filter.mouse_press(btn, mods), self.start_tracking
        )
        filter.add_event(
            self.state0, filter.mouse_move(btn, mods), self.move, self.state0
        )
        filter.add_event(
            self.state0,
            filter.mouse_release(btn, mods),
            self.stop_tracking,
            start_state,
        )
        filter.add_event(
            start_state, StandardKeyMatch(QKeySequence.Undo), self.undo, start_state
        )
        filter.add_event(
            start_state, StandardKeyMatch(QKeySequence.Redo), self.redo, start_state
        )
        self.handle = None  # first mouse position
        self.inside = False
        self._active = None  # mouse position seen during last event
        self.last_pos = None
        self.unselection_pending = None

        self.undo_stack = [None]
        self.undo_index = 0
        self.first_pos = None
        self.undo_action = None

    @property
    def active(self):
        if self._active is not None:
            return self._active()

    @active.setter
    def active(self, value):
        if value is None:
            self._active = None
        else:
            self._active = weakref.ref(value)

    def add_undo_move_action(self, undo_action):
        self.undo_stack = self.undo_stack[: self.undo_index + 1]
        self.undo_stack.append(undo_action)
        self.undo_index = len(self.undo_stack) - 1

    def undo(self, filter, event):
        action = self.undo_stack[self.undo_index]
        if action is not None:
            if action.is_valid():
                action.undo()
                filter.plot.replot()
            else:
                self.undo_stack.remove(action)
            self.undo_index -= 1

    def redo(self, filter, event):
        if self.undo_index < len(self.undo_stack) - 1:
            action = self.undo_stack[self.undo_index + 1]
            if action.is_valid():
                action.redo()
                filter.plot.replot()
                self.undo_index += 1
            else:
                self.undo_stack.remove(action)

    def start_tracking(self, filter, event):
        plot = filter.plot
        self.inside = False
        self.active = None
        self.handle = None
        self.first_pos = pos = event.pos()
        self.last_pos = QPointF(pos)
        selected = plot.get_active_item()
        distance = CONF.get("plot", "selection/distance", 6)

        (
            nearest,
            nearest_dist,
            nearest_handle,
            nearest_inside,
        ) = plot.get_nearest_object(pos, distance)
        if nearest is not None:
            # Is the nearest object the real deal?
            if not nearest.can_select() or nearest_dist >= distance:
                # Looking for the nearest object in z containing cursor position
                (
                    nearest,
                    nearest_dist,
                    nearest_handle,
                    nearest_inside,
                ) = plot.get_nearest_object_in_z(pos)

        # This will unselect active item only if it's not moved afterwards:
        self.unselection_pending = selected is nearest
        if selected and not self.multiselection:
            # An item is selected
            self.active = selected
            (dist, self.handle, self.inside, other_object) = self.active.hit_test(pos)
            if other_object is not None:
                # e.g. LegendBoxItem: 'other_object' is the selected curve
                plot.set_active_item(other_object)
                return
            if dist >= distance and not self.inside:
                # The following allows to move together selected items by
                # clicking inside any of them (instead of active item only)
                other_selitems = [
                    _it
                    for _it in plot.get_selected_items()
                    if _it is not self.active and _it.can_move()
                ]
                for selitem in other_selitems:
                    dist, handle, inside, _other = selitem.hit_test(pos)
                    if dist < distance or inside:
                        self.inside = inside
                        break
                else:
                    self.__unselect_objects(filter)
                    filter.set_state(self.start_state, event)
                    return
        else:
            # No item is selected
            self.active = nearest
            self.handle = nearest_handle
            self.inside = nearest_inside
            dist = nearest_dist
            if nearest is not None:
                plot.set_active_item(nearest)
                if not nearest.selected:
                    if not self.multiselection:
                        plot.unselect_all()
                    plot.select_item(nearest)

        # Eventually move or resize selected object:
        self.__move_or_resize_object(dist, distance, event, filter)
        plot.replot()

    def __unselect_objects(self, filter):
        """Unselect selected object*s*"""
        plot = filter.plot
        plot.unselect_all()
        plot.replot()

    def __move_or_resize_object(self, dist, distance, event, filter):
        if (
            dist < distance
            and self.handle is not None
            and (self.active.can_resize() or self.active.can_rotate())
        ):
            # Resize / move handle
            self.inside = False
            return
        if self.inside and self.active.can_move():
            # Move object
            return
        # can't move, can't resize
        filter.set_state(self.start_state, event)
        if self.unselection_pending:
            self.__unselect_objects(filter)

    def move(self, filter, event):
        if self.active is None:
            return
        self.unselection_pending = False
        if self.inside:
            self.active.move_local_shape(self.last_pos, event.pos())
            self.undo_action = UndoMoveObject(self.active, event.pos(), self.first_pos)
        else:
            ctrl = event.modifiers() & Qt.ControlModifier == Qt.ControlModifier
            self.active.move_local_point_to(self.handle, event.pos(), ctrl)
            self.undo_action = UndoMovePoint(
                self.active, self.first_pos, event.pos(), self.handle, ctrl
            )
        self.last_pos = QPointF(event.pos())
        filter.plot.replot()

    def stop_tracking(self, filter, event):
        self.add_undo_move_action(self.undo_action)
        if self.unselection_pending:
            self.__unselect_objects(filter)
        self.handle = None
        self.active = None


class RectangularSelectionHandler(DragHandler):

    #: Signal emitted by RectangularSelectionHandler when ending selection
    SIG_END_RECT = Signal(object, "QPointF", "QPointF")

    def __init__(self, filter, btn, mods=Qt.NoModifier, start_state=0):
        super(RectangularSelectionHandler, self).__init__(
            filter, btn, mods, start_state
        )
        self.avoid_null_shape = False

    def set_shape(self, shape, h0, h1, setup_shape_cb=None, avoid_null_shape=False):
        self.shape = shape
        self.shape_h0 = h0
        self.shape_h1 = h1
        self.setup_shape_cb = setup_shape_cb
        self.avoid_null_shape = avoid_null_shape

    def start_tracking(self, filter, event):
        self.start = self.last = QPointF(event.pos())

    def start_moving(self, filter, event):
        self.shape.attach(filter.plot)
        self.shape.setZ(filter.plot.get_max_z() + 1)
        if self.avoid_null_shape:
            self.start -= QPointF(1, 1)
        self.shape.move_local_point_to(self.shape_h0, self.start)
        self.shape.move_local_point_to(self.shape_h1, event.pos())
        self.start_moving_action(filter, event)
        if self.setup_shape_cb is not None:
            self.setup_shape_cb(self.shape)
        self.shape.show()
        filter.plot.replot()

    def start_moving_action(self, filter, event):
        """Les classes derivees peuvent surcharger cette methode"""
        pass

    def move(self, filter, event):
        self.shape.move_local_point_to(self.shape_h1, event.pos())
        self.move_action(filter, event)
        filter.plot.replot()

    def move_action(self, filter, event):
        """Les classes derivees peuvent surcharger cette methode"""
        pass

    def stop_moving(self, filter, event):
        self.shape.detach()
        self.stop_moving_action(filter, event)
        filter.plot.replot()

    def stop_moving_action(self, filter, event):
        """Les classes derivees peuvent surcharger cette methode"""
        self.SIG_END_RECT.emit(filter, self.start, event.pos())


class PointSelectionHandler(RectangularSelectionHandler):
    def stop_notmoving(self, filter, event):
        self.stop_moving(filter, event)


class ZoomRectHandler(RectangularSelectionHandler):
    def stop_moving_action(self, filter, event):
        filter.plot.do_zoom_rect_view(self.start, event.pos())


def setup_standard_tool_filter(filter, start_state):
    """Création des filtres standard (pan/zoom) sur boutons milieu/droit"""
    # Bouton du milieu
    PanHandler(filter, Qt.MidButton, start_state=start_state)
    AutoZoomHandler(filter, Qt.MidButton, start_state=start_state)

    # Bouton droit
    ZoomHandler(filter, Qt.RightButton, start_state=start_state)
    MenuHandler(filter, Qt.RightButton, start_state=start_state)

    # Autres (touches, move)
    MoveHandler(filter, start_state=start_state)
    MoveHandler(filter, start_state=start_state, mods=Qt.ShiftModifier)
    MoveHandler(filter, start_state=start_state, mods=Qt.AltModifier)
    return start_state



# -*- coding: utf-8 -*-
"""
guiqwt
======

Based on `PythonQwt` (plotting widgets for Qt graphical user interfaces) and
on the scientific modules NumPy and SciPy, `guiqwt` is a Python library
providing efficient 2D data-plotting features (curve/image visualization
and related tools) for interactive computing and signal/image processing
application development.

.. image:: images/panorama.png


External resources:
    * Python Package Index: `PyPI`_
    * Bug reports and feature requests: `GitHub`_
    * Help, support and discussions around the project: `GoogleGroup`_

.. _PyPI: https://pypi.python.org/pypi/guiqwt
.. _GitHub: https://github.com/PierreRaybaut/guiqwt
.. _GoogleGroup: http://groups.google.fr/group/guidata_guiqwt
"""


__version__ = "4.3.1"  # Update here *AND* in setup.py!
# (Until setup.py has been fully retrofitted, this manual sync is mandatory)


def about(html=True, copyright_only=False):
    """Return text about this package"""
    import sys, os, os.path as osp, platform, guidata, guiqwt, qwt
    from guiqwt.config import _
    from qtpy import QtCore as QC

    name = __file__.split(osp.sep)[-2]
    tf1 = (
        name,
        __version__,
        "guiqwt is a set of tools for curve and image plotting "
        "(extension to PythonQwt)",
    )
    tf2 = (
        platform.python_version(),
        "64 bits" if sys.maxsize > 2**32 else "32 bits",
        QC.__version__,
        qwt.__version__,
        guidata.__version__,
        guiqwt.__version__,
        _("on"),
        platform.system(),
    )
    if html:
        short_desc = (
            "This widget is powered by <b>%s</b> v%s"
            "<p>%s<p>Created by Pierre Raybaut" % tf1
        )
        desc = (
            "Copyright &copy; 2010-2016 CEA"
            "<p>Python %s %s, Qt %s, "
            "PythonQwt %s, guidata %s, guiqwt %s %s %s" % tf2
        )
        if not copyright_only:
            desc = short_desc + "<br>" + desc
    else:
        short_desc = (
            """%s v%s : %s
Created by Pierre Raybaut"""
            % tf1
        )
        desc = (
            """Copyright (c) 2010-2016 CEA
Python %s %s, Qt %s, PythonQwt %s, guidata %s, guiqwt %s %s %s"""
            % tf2
        )
        if not copyright_only:
            desc = short_desc + os.linesep + desc
    return desc


# Dear (Debian, RPM, ...) package makers, please feel free to customize the
# following path to module's data (images) and translations:
DATAPATH = LOCALEPATH = ""

# XXX: image.py: BaseImageItem.get_histogram: histo1 or histo2 ???
#     one version is faster but currently buggy for integers


#    Copyright © 2009-2015 CEA
#    Pierre Raybaut
#    Licensed under the terms of the CECILL License (see below)


#    LICENSE TERMS:
#    -------------
#
#    CeCILL FREE SOFTWARE LICENSE AGREEMENT
#
#
#        Notice
#
#    This Agreement is a Free Software license agreement that is the result
#    of discussions between its authors in order to ensure compliance with
#    the two main principles guiding its drafting:
#
#        * firstly, compliance with the principles governing the distribution
#          of Free Software: access to source code, broad rights granted to
#          users,
#        * secondly, the election of a governing law, French law, with which
#          it is conformant, both as regards the law of torts and
#          intellectual property law, and the protection that it offers to
#          both authors and holders of the economic rights over software.
#
#    The authors of the CeCILL (for Ce[a] C[nrs] I[nria] L[ogiciel] L[ibre])
#    license are:
#
#    Commissariat à l'Energie Atomique - CEA, a public scientific, technical
#    and industrial research establishment, having its principal place of
#    business at 25 rue Leblanc, immeuble Le Ponant D, 75015 Paris, France.
#
#    Centre National de la Recherche Scientifique - CNRS, a public scientific
#    and technological establishment, having its principal place of business
#    at 3 rue Michel-Ange, 75794 Paris cedex 16, France.
#
#    Institut National de Recherche en Informatique et en Automatique -
#    INRIA, a public scientific and technological establishment, having its
#    principal place of business at Domaine de Voluceau, Rocquencourt, BP
#    105, 78153 Le Chesnay cedex, France.
#
#
#        Preamble
#
#    The purpose of this Free Software license agreement is to grant users
#    the right to modify and redistribute the software governed by this
#    license within the framework of an open source distribution model.
#
#    The exercising of these rights is conditional upon certain obligations
#    for users so as to preserve this status for all subsequent redistributions.
#
#    In consideration of access to the source code and the rights to copy,
#    modify and redistribute granted by the license, users are provided only
#    with a limited warranty and the software's author, the holder of the
#    economic rights, and the successive licensors only have limited liability.
#
#    In this respect, the risks associated with loading, using, modifying
#    and/or developing or reproducing the software by the user are brought to
#    the user's attention, given its Free Software status, which may make it
#    complicated to use, with the result that its use is reserved for
#    developers and experienced professionals having in-depth computer
#    knowledge. Users are therefore encouraged to load and test the
#    suitability of the software as regards their requirements in conditions
#    enabling the security of their systems and/or data to be ensured and,
#    more generally, to use and operate it in the same conditions of
#    security. This Agreement may be freely reproduced and published,
#    provided it is not altered, and that no provisions are either added or
#    removed herefrom.
#
#    This Agreement may apply to any or all software for which the holder of
#    the economic rights decides to submit the use thereof to its provisions.
#
#
#        Article 1 - DEFINITIONS
#
#    For the purpose of this Agreement, when the following expressions
#    commence with a capital letter, they shall have the following meaning:
#
#    Agreement: means this license agreement, and its possible subsequent
#    versions and annexes.
#
#    Software: means the software in its Object Code and/or Source Code form
#    and, where applicable, its documentation, "as is" when the Licensee
#    accepts the Agreement.
#
#    Initial Software: means the Software in its Source Code and possibly its
#    Object Code form and, where applicable, its documentation, "as is" when
#    it is first distributed under the terms and conditions of the Agreement.
#
#    Modified Software: means the Software modified by at least one
#    Contribution.
#
#    Source Code: means all the Software's instructions and program lines to
#    which access is required so as to modify the Software.
#
#    Object Code: means the binary files originating from the compilation of
#    the Source Code.
#
#    Holder: means the holder(s) of the economic rights over the Initial
#    Software.
#
#    Licensee: means the Software user(s) having accepted the Agreement.
#
#    Contributor: means a Licensee having made at least one Contribution.
#
#    Licensor: means the Holder, or any other individual or legal entity, who
#    distributes the Software under the Agreement.
#
#    Contribution: means any or all modifications, corrections, translations,
#    adaptations and/or new functions integrated into the Software by any or
#    all Contributors, as well as any or all Internal Modules.
#
#    Module: means a set of sources files including their documentation that
#    enables supplementary functions or services in addition to those offered
#    by the Software.
#
#    External Module: means any or all Modules, not derived from the
#    Software, so that this Module and the Software run in separate address
#    spaces, with one calling the other when they are run.
#
#    Internal Module: means any or all Module, connected to the Software so
#    that they both execute in the same address space.
#
#    GNU GPL: means the GNU General Public License version 2 or any
#    subsequent version, as published by the Free Software Foundation Inc.
#
#    Parties: mean both the Licensee and the Licensor.
#
#    These expressions may be used both in singular and plural form.
#
#
#        Article 2 - PURPOSE
#
#    The purpose of the Agreement is the grant by the Licensor to the
#    Licensee of a non-exclusive, transferable and worldwide license for the
#    Software as set forth in Article 5 hereinafter for the whole term of the
#    protection granted by the rights over said Software.
#
#
#        Article 3 - ACCEPTANCE
#
#    3.1 The Licensee shall be deemed as having accepted the terms and
#    conditions of this Agreement upon the occurrence of the first of the
#    following events:
#
#        * (i) loading the Software by any or all means, notably, by
#          downloading from a remote server, or by loading from a physical
#          medium;
#        * (ii) the first time the Licensee exercises any of the rights
#          granted hereunder.
#
#    3.2 One copy of the Agreement, containing a notice relating to the
#    characteristics of the Software, to the limited warranty, and to the
#    fact that its use is restricted to experienced users has been provided
#    to the Licensee prior to its acceptance as set forth in Article 3.1
#    hereinabove, and the Licensee hereby acknowledges that it has read and
#    understood it.
#
#
#        Article 4 - EFFECTIVE DATE AND TERM
#
#
#          4.1 EFFECTIVE DATE
#
#    The Agreement shall become effective on the date when it is accepted by
#    the Licensee as set forth in Article 3.1.
#
#
#          4.2 TERM
#
#    The Agreement shall remain in force for the entire legal term of
#    protection of the economic rights over the Software.
#
#
#        Article 5 - SCOPE OF RIGHTS GRANTED
#
#    The Licensor hereby grants to the Licensee, who accepts, the following
#    rights over the Software for any or all use, and for the term of the
#    Agreement, on the basis of the terms and conditions set forth hereinafter.
#
#    Besides, if the Licensor owns or comes to own one or more patents
#    protecting all or part of the functions of the Software or of its
#    components, the Licensor undertakes not to enforce the rights granted by
#    these patents against successive Licensees using, exploiting or
#    modifying the Software. If these patents are transferred, the Licensor
#    undertakes to have the transferees subscribe to the obligations set
#    forth in this paragraph.
#
#
#          5.1 RIGHT OF USE
#
#    The Licensee is authorized to use the Software, without any limitation
#    as to its fields of application, with it being hereinafter specified
#    that this comprises:
#
#       1. permanent or temporary reproduction of all or part of the Software
#          by any or all means and in any or all form.
#
#       2. loading, displaying, running, or storing the Software on any or
#          all medium.
#
#       3. entitlement to observe, study or test its operation so as to
#          determine the ideas and principles behind any or all constituent
#          elements of said Software. This shall apply when the Licensee
#          carries out any or all loading, displaying, running, transmission
#          or storage operation as regards the Software, that it is entitled
#          to carry out hereunder.
#
#
#          5.2 ENTITLEMENT TO MAKE CONTRIBUTIONS
#
#    The right to make Contributions includes the right to translate, adapt,
#    arrange, or make any or all modifications to the Software, and the right
#    to reproduce the resulting software.
#
#    The Licensee is authorized to make any or all Contributions to the
#    Software provided that it includes an explicit notice that it is the
#    author of said Contribution and indicates the date of the creation thereof.
#
#
#          5.3 RIGHT OF DISTRIBUTION
#
#    In particular, the right of distribution includes the right to publish,
#    transmit and communicate the Software to the general public on any or
#    all medium, and by any or all means, and the right to market, either in
#    consideration of a fee, or free of charge, one or more copies of the
#    Software by any means.
#
#    The Licensee is further authorized to distribute copies of the modified
#    or unmodified Software to third parties according to the terms and
#    conditions set forth hereinafter.
#
#
#            5.3.1 DISTRIBUTION OF SOFTWARE WITHOUT MODIFICATION
#
#    The Licensee is authorized to distribute true copies of the Software in
#    Source Code or Object Code form, provided that said distribution
#    complies with all the provisions of the Agreement and is accompanied by:
#
#       1. a copy of the Agreement,
#
#       2. a notice relating to the limitation of both the Licensor's
#          warranty and liability as set forth in Articles 8 and 9,
#
#    and that, in the event that only the Object Code of the Software is
#    redistributed, the Licensee allows future Licensees unhindered access to
#    the full Source Code of the Software by indicating how to access it, it
#    being understood that the additional cost of acquiring the Source Code
#    shall not exceed the cost of transferring the data.
#
#
#            5.3.2 DISTRIBUTION OF MODIFIED SOFTWARE
#
#    When the Licensee makes a Contribution to the Software, the terms and
#    conditions for the distribution of the resulting Modified Software
#    become subject to all the provisions of this Agreement.
#
#    The Licensee is authorized to distribute the Modified Software, in
#    source code or object code form, provided that said distribution
#    complies with all the provisions of the Agreement and is accompanied by:
#
#       1. a copy of the Agreement,
#
#       2. a notice relating to the limitation of both the Licensor's
#          warranty and liability as set forth in Articles 8 and 9,
#
#    and that, in the event that only the object code of the Modified
#    Software is redistributed, the Licensee allows future Licensees
#    unhindered access to the full source code of the Modified Software by
#    indicating how to access it, it being understood that the additional
#    cost of acquiring the source code shall not exceed the cost of
#    transferring the data.
#
#
#            5.3.3 DISTRIBUTION OF EXTERNAL MODULES
#
#    When the Licensee has developed an External Module, the terms and
#    conditions of this Agreement do not apply to said External Module, that
#    may be distributed under a separate license agreement.
#
#
#            5.3.4 COMPATIBILITY WITH THE GNU GPL
#
#    The Licensee can include a code that is subject to the provisions of one
#    of the versions of the GNU GPL in the Modified or unmodified Software,
#    and distribute that entire code under the terms of the same version of
#    the GNU GPL.
#
#    The Licensee can include the Modified or unmodified Software in a code
#    that is subject to the provisions of one of the versions of the GNU GPL,
#    and distribute that entire code under the terms of the same version of
#    the GNU GPL.
#
#
#        Article 6 - INTELLECTUAL PROPERTY
#
#
#          6.1 OVER THE INITIAL SOFTWARE
#
#    The Holder owns the economic rights over the Initial Software. Any or
#    all use of the Initial Software is subject to compliance with the terms
#    and conditions under which the Holder has elected to distribute its work
#    and no one shall be entitled to modify the terms and conditions for the
#    distribution of said Initial Software.
#
#    The Holder undertakes that the Initial Software will remain ruled at
#    least by this Agreement, for the duration set forth in Article 4.2.
#
#
#          6.2 OVER THE CONTRIBUTIONS
#
#    The Licensee who develops a Contribution is the owner of the
#    intellectual property rights over this Contribution as defined by
#    applicable law.
#
#
#          6.3 OVER THE EXTERNAL MODULES
#
#    The Licensee who develops an External Module is the owner of the
#    intellectual property rights over this External Module as defined by
#    applicable law and is free to choose the type of agreement that shall
#    govern its distribution.
#
#
#          6.4 JOINT PROVISIONS
#
#    The Licensee expressly undertakes:
#
#       1. not to remove, or modify, in any manner, the intellectual property
#          notices attached to the Software;
#
#       2. to reproduce said notices, in an identical manner, in the copies
#          of the Software modified or not.
#
#    The Licensee undertakes not to directly or indirectly infringe the
#    intellectual property rights of the Holder and/or Contributors on the
#    Software and to take, where applicable, vis-à-vis its staff, any and all
#    measures required to ensure respect of said intellectual property rights
#    of the Holder and/or Contributors.
#
#
#        Article 7 - RELATED SERVICES
#
#    7.1 Under no circumstances shall the Agreement oblige the Licensor to
#    provide technical assistance or maintenance services for the Software.
#
#    However, the Licensor is entitled to offer this type of services. The
#    terms and conditions of such technical assistance, and/or such
#    maintenance, shall be set forth in a separate instrument. Only the
#    Licensor offering said maintenance and/or technical assistance services
#    shall incur liability therefor.
#
#    7.2 Similarly, any Licensor is entitled to offer to its licensees, under
#    its sole responsibility, a warranty, that shall only be binding upon
#    itself, for the redistribution of the Software and/or the Modified
#    Software, under terms and conditions that it is free to decide. Said
#    warranty, and the financial terms and conditions of its application,
#    shall be subject of a separate instrument executed between the Licensor
#    and the Licensee.
#
#
#        Article 8 - LIABILITY
#
#    8.1 Subject to the provisions of Article 8.2, the Licensee shall be
#    entitled to claim compensation for any direct loss it may have suffered
#    from the Software as a result of a fault on the part of the relevant
#    Licensor, subject to providing evidence thereof.
#
#    8.2 The Licensor's liability is limited to the commitments made under
#    this Agreement and shall not be incurred as a result of in particular:
#    (i) loss due the Licensee's total or partial failure to fulfill its
#    obligations, (ii) direct or consequential loss that is suffered by the
#    Licensee due to the use or performance of the Software, and (iii) more
#    generally, any consequential loss. In particular the Parties expressly
#    agree that any or all pecuniary or business loss (i.e. loss of data,
#    loss of profits, operating loss, loss of customers or orders,
#    opportunity cost, any disturbance to business activities) or any or all
#    legal proceedings instituted against the Licensee by a third party,
#    shall constitute consequential loss and shall not provide entitlement to
#    any or all compensation from the Licensor.
#
#
#        Article 9 - WARRANTY
#
#    9.1 The Licensee acknowledges that the scientific and technical
#    state-of-the-art when the Software was distributed did not enable all
#    possible uses to be tested and verified, nor for the presence of
#    possible defects to be detected. In this respect, the Licensee's
#    attention has been drawn to the risks associated with loading, using,
#    modifying and/or developing and reproducing the Software which are
#    reserved for experienced users.
#
#    The Licensee shall be responsible for verifying, by any or all means,
#    the suitability of the product for its requirements, its good working
#    order, and for ensuring that it shall not cause damage to either persons
#    or properties.
#
#    9.2 The Licensor hereby represents, in good faith, that it is entitled
#    to grant all the rights over the Software (including in particular the
#    rights set forth in Article 5).
#
#    9.3 The Licensee acknowledges that the Software is supplied "as is" by
#    the Licensor without any other express or tacit warranty, other than
#    that provided for in Article 9.2 and, in particular, without any warranty
#    as to its commercial value, its secured, safe, innovative or relevant
#    nature.
#
#    Specifically, the Licensor does not warrant that the Software is free
#    from any error, that it will operate without interruption, that it will
#    be compatible with the Licensee's own equipment and software
#    configuration, nor that it will meet the Licensee's requirements.
#
#    9.4 The Licensor does not either expressly or tacitly warrant that the
#    Software does not infringe any third party intellectual property right
#    relating to a patent, software or any other property right. Therefore,
#    the Licensor disclaims any and all liability towards the Licensee
#    arising out of any or all proceedings for infringement that may be
#    instituted in respect of the use, modification and redistribution of the
#    Software. Nevertheless, should such proceedings be instituted against
#    the Licensee, the Licensor shall provide it with technical and legal
#    assistance for its defense. Such technical and legal assistance shall be
#    decided on a case-by-case basis between the relevant Licensor and the
#    Licensee pursuant to a memorandum of understanding. The Licensor
#    disclaims any and all liability as regards the Licensee's use of the
#    name of the Software. No warranty is given as regards the existence of
#    prior rights over the name of the Software or as regards the existence
#    of a trademark.
#
#
#        Article 10 - TERMINATION
#
#    10.1 In the event of a breach by the Licensee of its obligations
#    hereunder, the Licensor may automatically terminate this Agreement
#    thirty (30) days after notice has been sent to the Licensee and has
#    remained ineffective.
#
#    10.2 A Licensee whose Agreement is terminated shall no longer be
#    authorized to use, modify or distribute the Software. However, any
#    licenses that it may have granted prior to termination of the Agreement
#    shall remain valid subject to their having been granted in compliance
#    with the terms and conditions hereof.
#
#
#        Article 11 - MISCELLANEOUS
#
#
#          11.1 EXCUSABLE EVENTS
#
#    Neither Party shall be liable for any or all delay, or failure to
#    perform the Agreement, that may be attributable to an event of force
#    majeure, an act of God or an outside cause, such as defective
#    functioning or interruptions of the electricity or telecommunications
#    networks, network paralysis following a virus attack, intervention by
#    government authorities, natural disasters, water damage, earthquakes,
#    fire, explosions, strikes and labor unrest, war, etc.
#
#    11.2 Any failure by either Party, on one or more occasions, to invoke
#    one or more of the provisions hereof, shall under no circumstances be
#    interpreted as being a waiver by the interested Party of its right to
#    invoke said provision(s) subsequently.
#
#    11.3 The Agreement cancels and replaces any or all previous agreements,
#    whether written or oral, between the Parties and having the same
#    purpose, and constitutes the entirety of the agreement between said
#    Parties concerning said purpose. No supplement or modification to the
#    terms and conditions hereof shall be effective as between the Parties
#    unless it is made in writing and signed by their duly authorized
#    representatives.
#
#    11.4 In the event that one or more of the provisions hereof were to
#    conflict with a current or future applicable act or legislative text,
#    said act or legislative text shall prevail, and the Parties shall make
#    the necessary amendments so as to comply with said act or legislative
#    text. All other provisions shall remain effective. Similarly, invalidity
#    of a provision of the Agreement, for any reason whatsoever, shall not
#    cause the Agreement as a whole to be invalid.
#
#
#          11.5 LANGUAGE
#
#    The Agreement is drafted in both French and English and both versions
#    are deemed authentic.
#
#
#        Article 12 - NEW VERSIONS OF THE AGREEMENT
#
#    12.1 Any person is authorized to duplicate and distribute copies of this
#    Agreement.
#
#    12.2 So as to ensure coherence, the wording of this Agreement is
#    protected and may only be modified by the authors of the License, who
#    reserve the right to periodically publish updates or new versions of the
#    Agreement, each with a separate number. These subsequent versions may
#    address new issues encountered by Free Software.
#
#    12.3 Any Software distributed under a given version of the Agreement may
#    only be subsequently distributed under the same version of the Agreement
#    or a subsequent version, subject to the provisions of Article 5.3.4.
#
#
#        Article 13 - GOVERNING LAW AND JURISDICTION
#
#    13.1 The Agreement is governed by French law. The Parties agree to
#    endeavor to seek an amicable solution to any disagreements or disputes
#    that may arise during the performance of the Agreement.
#
#    13.2 Failing an amicable solution within two (2) months as from their
#    occurrence, and unless emergency proceedings are necessary, the
#    disagreements or disputes shall be referred to the Paris Courts having
#    jurisdiction, by the more diligent Party.
#
#
#    Version 2.0 dated 2006-09-05.



# -*- coding: utf-8 -*-
#
# Copyright © 2011 CEA
# Ludovic Aubry
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""PolygonMapItem test

PolygonMapItem is intended to display maps ie items containing
several hundreds of independent polygons.
"""


SHOW = True  # Show test in GUI-based test launcher

from guiqwt.plot import ImageDialog
from guiqwt.curve import PolygonMapItem

from numpy.random import rand, randint
from numpy import concatenate, linspace, int32, uint32, zeros, empty, pi, cos, sin


# Create a sample dataset consisting of tesselated circles randomly placed
# in a box
RMAX = 0.5
XMAX = YMAX = 10.0
NSEGMIN = 4
NSEGMAX = 300


def create_circle():
    x, y, rmax = rand(3)
    rmax *= RMAX
    x *= XMAX
    y *= YMAX
    nseg = randint(NSEGMIN, NSEGMAX)
    th = linspace(0, 2 * pi, nseg)
    PTS = empty((nseg, 2), float)
    PTS[:, 0] = x + rmax * cos(th)
    PTS[:, 1] = y + rmax * sin(th)
    return PTS


NCIRC = 1000
COLORS = [
    (0xFF000000, 0x8000FF00),
    (0xFF0000FF, 0x800000FF),
    (0xFF000000, 0x80FF0000),
    (0xFF00FF00, 0x80000000),
]


def test():
    win = ImageDialog(edit=True, toolbar=True, wintitle="Sample multi-polygon item")
    plot = win.get_plot()
    plot.set_aspect_ratio(lock=True)
    plot.set_antialiasing(False)
    plot.set_axis_direction("left", False)
    plot.set_axis_title("bottom", "Lon")
    plot.set_axis_title("left", "Lat")

    points = []
    offsets = zeros((NCIRC, 2), int32)
    colors = zeros((NCIRC, 2), uint32)
    npts = 0
    for k in range(NCIRC):
        pts = create_circle()
        offsets[k, 0] = k
        offsets[k, 1] = npts
        npts += pts.shape[0]
        points.append(pts)
        colors[k, 0] = COLORS[k % len(COLORS)][0]
        colors[k, 1] = COLORS[(3 * k) % len(COLORS)][1]
    points = concatenate(points)

    print(NCIRC, "Polygons")
    print(points.shape[0], "Points")

    crv = PolygonMapItem()
    crv.set_data(points, offsets, colors)
    plot.add_item(crv, z=0)
    win.show()
    win.exec_()


if __name__ == "__main__":
    import guidata

    app = guidata.qapplication()
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2012 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Resize test: using the scaler C++ engine to resize images"""

SHOW = True  # Show test in GUI-based test launcher


def test():
    """Test"""
    import os.path as osp
    from guiqwt import io, scaler, pyplot as plt

    filename = osp.join(osp.dirname(__file__), "brain.png")
    data = io.imread(filename)
    dst_image = scaler.resize(data, (2000, 3000))

    plt.imshow(dst_image, interpolation="nearest")
    plt.show()


if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-
#
# Copyright © 2012 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""2-D Histogram (with function) test"""

# SHOW = True # Show test in GUI-based test launcher
# TODO: change this test so that shown data means something...

from numpy import random, array, dot, concatenate

from guiqwt.plot import ImageDialog
from guiqwt.builder import make
from guiqwt.config import _


def hist2d_func(X, Y, Z):
    win = ImageDialog(
        edit=True, toolbar=True, wintitle="2-D Histogram X0=(0,1), X1=(-1,-1)"
    )
    hist2d = make.histogram2D(X, Y, 200, 200, Z=Z, computation=2)
    curve = make.curve(X[::50], Y[::50], linestyle="", marker="+", title=_("Markers"))
    plot = win.get_plot()
    plot.set_aspect_ratio(lock=False)
    plot.set_antialiasing(False)
    plot.add_item(hist2d)
    plot.add_item(curve)
    plot.set_item_visible(curve, False)
    win.show()
    win.exec_()


if __name__ == "__main__":
    import guidata

    _app = guidata.qapplication()
    N = 150000
    m = array([[1.0, 0.2], [-0.2, 3.0]])
    X1 = random.normal(0, 0.3, size=(N, 2))
    X2 = random.normal(0, 0.3, size=(N, 2))
    X = concatenate((X1 + [0, 1.0], dot(X2, m) + [-1, -1.0]))
    hist2d_func(X[:, 0], X[:, 1], X[:, 0] + X[:, 1])



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""
guiqwt.config
-------------

The `config` module handles `guiqwt` configuration (options, images and icons).
"""

import os.path as osp

from guidata.configtools import add_image_module_path, get_translation
from guidata import qthelpers

IS_DARK = qthelpers.is_dark_mode()
MAIN_FG_COLOR = "white" if IS_DARK else "black"
MAIN_BG_COLOR = "#222222" if IS_DARK else "white"

_ = get_translation("guiqwt")


def make_title(basename, count):
    """Make item title with *basename* and *count* number"""
    return "%s %s%d" % (basename, _("#"), count)


APP_PATH = osp.dirname(__file__)
add_image_module_path("guiqwt", "images")

DEFAULTS = {
    "plot": {
        "selection/distance": 6,
        "antialiasing": False,
        "title/font/size": 12,
        "title/font/bold": False,
        "selected_curve_symbol/marker": "Rect",
        "selected_curve_symbol/edgecolor": "gray",
        "selected_curve_symbol/facecolor": MAIN_FG_COLOR,
        "selected_curve_symbol/alpha": 0.3,
        "selected_curve_symbol/size": 7,
        # Default parameters for plot axes
        "axis/title": "",
        "axis/unit": "",
        "axis/color": MAIN_FG_COLOR,
        "axis/title_font/size": 8,
        "axis/title_font/family": "default",
        "axis/title_font/bold": False,
        "axis/title_font/italic": False,
        "axis/ticks_font/size": 8,
        "axis/ticks_font/family": "default",
        "axis/ticks_font/bold": False,
        "axis/ticks_font/italic": False,
        # Default parameters for X/Y image axes
        "image_axis/title": "",
        "image_axis/unit": _("pixels"),
        "image_axis/color": MAIN_FG_COLOR,
        "image_axis/title_font/size": 8,
        "image_axis/title_font/family": "default",
        "image_axis/title_font/bold": False,
        "image_axis/title_font/italic": False,
        "image_axis/ticks_font/size": 8,
        "image_axis/ticks_font/family": "default",
        "image_axis/ticks_font/bold": False,
        "image_axis/ticks_font/italic": False,
        # Default parameters for color scale
        "color_axis/title": _("Intensity"),
        "color_axis/unit": _("lsb"),
        "color_axis/color": MAIN_FG_COLOR,
        "color_axis/title_font/size": 8,
        "color_axis/title_font/family": "default",
        "color_axis/title_font/bold": False,
        "color_axis/title_font/italic": False,
        "color_axis/ticks_font/size": 8,
        "color_axis/ticks_font/family": "default",
        "color_axis/ticks_font/bold": False,
        "color_axis/ticks_font/italic": False,
        "grid/background": MAIN_BG_COLOR,
        "grid/maj_xenabled": True,
        "grid/maj_yenabled": True,
        "grid/maj_line/color": "darkgray",
        "grid/maj_line/width": 1,
        "grid/maj_line/style": "DotLine",
        "grid/min_xenabled": True,
        "grid/min_yenabled": True,
        "grid/min_line/color": "#454545" if IS_DARK else "#eaeaea",
        "grid/min_line/width": 1,
        "grid/min_line/style": "DotLine",
        "marker/curve/symbol/marker": "Rect",
        "marker/curve/symbol/edgecolor": "blue",
        "marker/curve/symbol/facecolor": "cyan",
        "marker/curve/symbol/alpha": 0.8,
        "marker/curve/symbol/size": 7,
        "marker/curve/text/font/size": 8,
        "marker/curve/text/font/family": "default",
        "marker/curve/text/font/bold": False,
        "marker/curve/text/font/italic": False,
        "marker/curve/text/textcolor": "black",
        "marker/curve/text/background_color": "#ffffff",
        "marker/curve/text/background_alpha": 0.8,
        "marker/curve/line/style": "SolidLine",
        "marker/curve/line/color": MAIN_FG_COLOR,
        "marker/curve/line/width": 0,
        "marker/curve/sel_symbol/marker": "Rect",
        "marker/curve/sel_symbol/edgecolor": "blue",
        "marker/curve/sel_symbol/facecolor": "cyan",
        "marker/curve/sel_symbol/alpha": 0.8,
        "marker/curve/sel_symbol/size": 7,
        "marker/curve/sel_text/font/size": 8,
        "marker/curve/sel_text/font/family": "default",
        "marker/curve/sel_text/font/bold": False,
        "marker/curve/sel_text/font/italic": False,
        "marker/curve/sel_text/textcolor": "black",
        "marker/curve/sel_text/background_color": "#ffffff",
        "marker/curve/sel_text/background_alpha": 0.8,
        "marker/curve/sel_line/style": "SolidLine",
        "marker/curve/sel_line/color": MAIN_FG_COLOR,
        "marker/curve/sel_line/width": 0,
        "marker/curve/markerstyle": "NoLine",
        "marker/curve/spacing": 7,
        "marker/cross/symbol/marker": "Cross",
        "marker/cross/symbol/edgecolor": MAIN_FG_COLOR,
        "marker/cross/symbol/facecolor": "red",
        "marker/cross/symbol/alpha": 1.0,
        "marker/cross/symbol/size": 8,
        "marker/cross/text/font/family": "default",
        "marker/cross/text/font/size": 8,
        "marker/cross/text/font/bold": False,
        "marker/cross/text/font/italic": False,
        "marker/cross/text/textcolor": "black",
        "marker/cross/text/background_color": "#ffffff",
        "marker/cross/text/background_alpha": 0.8,
        "marker/cross/line/style": "DashLine",
        "marker/cross/line/color": "yellow",
        "marker/cross/line/width": 1,
        "marker/cross/sel_symbol/marker": "Cross",
        "marker/cross/sel_symbol/edgecolor": MAIN_FG_COLOR,
        "marker/cross/sel_symbol/facecolor": "red",
        "marker/cross/sel_symbol/alpha": 1.0,
        "marker/cross/sel_symbol/size": 8,
        "marker/cross/sel_text/font/family": "default",
        "marker/cross/sel_text/font/size": 8,
        "marker/cross/sel_text/font/bold": False,
        "marker/cross/sel_text/font/italic": False,
        "marker/cross/sel_text/textcolor": "black",
        "marker/cross/sel_text/background_color": "#ffffff",
        "marker/cross/sel_text/background_alpha": 0.8,
        "marker/cross/sel_line/style": "DashLine",
        "marker/cross/sel_line/color": "yellow",
        "marker/cross/sel_line/width": 1,
        "marker/cross/markerstyle": "Cross",
        "marker/cross/spacing": 7,
        "marker/selectpoint/symbol/marker": "Rect",
        "marker/selectpoint/symbol/edgecolor": "blue",
        "marker/selectpoint/symbol/facecolor": "cyan",
        "marker/selectpoint/symbol/alpha": 0.8,
        "marker/selectpoint/symbol/size": 7,
        "marker/selectpoint/text/font/size": 8,
        "marker/selectpoint/text/font/family": "default",
        "marker/selectpoint/text/font/bold": False,
        "marker/selectpoint/text/font/italic": False,
        "marker/selectpoint/text/textcolor": "black",
        "marker/selectpoint/text/background_color": "#ffffff",
        "marker/selectpoint/text/background_alpha": 0.8,
        "marker/selectpoint/line/style": "SolidLine",
        "marker/selectpoint/line/color": MAIN_FG_COLOR,
        "marker/selectpoint/line/width": 0,
        "marker/selectpoint/sel_symbol/marker": "Rect",
        "marker/selectpoint/sel_symbol/edgecolor": "blue",
        "marker/selectpoint/sel_symbol/facecolor": "cyan",
        "marker/selectpoint/sel_symbol/alpha": 0.8,
        "marker/selectpoint/sel_symbol/size": 7,
        "marker/selectpoint/sel_text/font/size": 8,
        "marker/selectpoint/sel_text/font/family": "default",
        "marker/selectpoint/sel_text/font/bold": False,
        "marker/selectpoint/sel_text/font/italic": False,
        "marker/selectpoint/sel_text/textcolor": "black",
        "marker/selectpoint/sel_text/background_color": "#ffffff",
        "marker/selectpoint/sel_text/background_alpha": 0.8,
        "marker/selectpoint/sel_line/style": "SolidLine",
        "marker/selectpoint/sel_line/color": MAIN_FG_COLOR,
        "marker/selectpoint/sel_line/width": 0,
        "marker/selectpoint/markerstyle": "NoLine",
        "marker/selectpoint/spacing": 7,
        "marker/cursor/line/style": "SolidLine",
        "marker/cursor/line/color": "#ff9393",
        "marker/cursor/line/width": 1.0,
        "marker/cursor/symbol/marker": "Ellipse",
        "marker/cursor/symbol/size": 7,
        "marker/cursor/symbol/edgecolor": MAIN_BG_COLOR,
        "marker/cursor/symbol/facecolor": "#ff9393",
        "marker/cursor/symbol/alpha": 1.0,
        "marker/cursor/text/font/size": 8,
        "marker/cursor/text/font/family": "default",
        "marker/cursor/text/font/bold": False,
        "marker/cursor/text/font/italic": False,
        "marker/cursor/text/textcolor": "#ff9393",
        "marker/cursor/text/background_color": "#ffffff",
        "marker/cursor/text/background_alpha": 0.8,
        "marker/cursor/sel_line/style": "SolidLine",
        "marker/cursor/sel_line/color": "red",
        "marker/cursor/sel_line/width": 2.0,
        "marker/cursor/sel_symbol/marker": "Ellipse",
        "marker/cursor/sel_symbol/size": 9,
        "marker/cursor/sel_symbol/edgecolor": MAIN_BG_COLOR,
        "marker/cursor/sel_symbol/facecolor": "red",
        "marker/cursor/sel_symbol/alpha": 0.9,
        "marker/cursor/sel_text/font/size": 8,
        "marker/cursor/sel_text/font/family": "default",
        "marker/cursor/sel_text/font/bold": True,
        "marker/cursor/sel_text/font/italic": False,
        "marker/cursor/sel_text/textcolor": "red",
        "marker/cursor/sel_text/background_color": "#ffffff",
        "marker/cursor/sel_text/background_alpha": 0.8,
        "marker/cursor/markerstyle": "NoLine",
        "marker/cursor/spacing": 7,
        "shape/drag/line/style": "SolidLine",
        "shape/drag/line/color": "#ffff00",
        "shape/drag/line/width": 1,
        "shape/drag/fill/style": "SolidPattern",
        "shape/drag/fill/color": MAIN_BG_COLOR,
        "shape/drag/fill/alpha": 0.1,
        "shape/drag/symbol/marker": "Rect",
        "shape/drag/symbol/size": 3,
        "shape/drag/symbol/edgecolor": "#ffff00",
        "shape/drag/symbol/facecolor": "#ffff00",
        "shape/drag/symbol/alpha": 1.0,
        "shape/drag/sel_line/style": "SolidLine",
        "shape/drag/sel_line/color": "#00ff00",
        "shape/drag/sel_line/width": 1,
        "shape/drag/sel_fill/style": "SolidPattern",
        "shape/drag/sel_fill/color": MAIN_BG_COLOR,
        "shape/drag/sel_fill/alpha": 0.1,
        "shape/drag/sel_symbol/marker": "Rect",
        "shape/drag/sel_symbol/size": 9,
        "shape/drag/sel_symbol/edgecolor": "#00aa00",
        "shape/drag/sel_symbol/facecolor": "#00ff00",
        "shape/drag/sel_symbol/alpha": 0.7,
        "shape/imageborder/line/style": "NoPen",
        "shape/imageborder/line/color": "gray",
        "shape/imageborder/line/width": 0,
        "shape/imageborder/fill/style": "NoBrush",
        "shape/imageborder/fill/color": "black",
        "shape/imageborder/fill/alpha": 0.0,
        "shape/imageborder/fill/angle": 0.0,
        "shape/imageborder/fill/sx": 1.0,
        "shape/imageborder/fill/sy": 1.0,
        "shape/imageborder/symbol/marker": "NoSymbol",
        "shape/imageborder/symbol/size": 7,
        "shape/imageborder/symbol/edgecolor": "gray",
        "shape/imageborder/symbol/facecolor": MAIN_FG_COLOR,
        "shape/imageborder/symbol/alpha": 0.3,
        "shape/imageborder/sel_line/style": "SolidLine",
        "shape/imageborder/sel_line/color": "gray",
        "shape/imageborder/sel_line/width": 3,
        "shape/imageborder/sel_symbol/marker": "Rect",
        "shape/imageborder/sel_symbol/size": 7,
        "shape/imageborder/sel_symbol/edgecolor": "gray",
        "shape/imageborder/sel_symbol/facecolor": MAIN_FG_COLOR,
        "shape/imageborder/sel_symbol/alpha": 0.8,
        "shape/imageborder/sel_fill/style": "NoBrush",
        "shape/imageborder/sel_fill/color": "gray",
        "shape/imageborder/sel_fill/alpha": 0.5,
        "shape/imageborder/sel_fill/angle": 0.0,
        "shape/imageborder/sel_fill/sx": 1.0,
        "shape/imageborder/sel_fill/sy": 1.0,
        "shape/imagefilter/line/style": "SolidLine",
        "shape/imagefilter/line/color": "#ffff00",
        "shape/imagefilter/line/width": 1,
        "shape/imagefilter/sel_line/style": "SolidLine",
        "shape/imagefilter/sel_line/color": "#00ffff",
        "shape/imagefilter/sel_line/width": 2,
        "shape/imagefilter/fill/style": "NoBrush",
        "shape/imagefilter/fill/color": MAIN_BG_COLOR,
        "shape/imagefilter/fill/alpha": 0.0,
        "shape/imagefilter/sel_fill/style": "SolidPattern",
        "shape/imagefilter/sel_fill/color": MAIN_BG_COLOR,
        "shape/imagefilter/sel_fill/alpha": 0.2,
        "shape/imagefilter/symbol/marker": "Rect",
        "shape/imagefilter/symbol/size": 3,
        "shape/imagefilter/symbol/edgecolor": "#ffff00",
        "shape/imagefilter/symbol/facecolor": "#ffff00",
        "shape/imagefilter/symbol/alpha": 1.0,
        "shape/imagefilter/sel_symbol/marker": "Ellipse",
        "shape/imagefilter/sel_symbol/size": 7,
        "shape/imagefilter/sel_symbol/edgecolor": "#0000ff",
        "shape/imagefilter/sel_symbol/facecolor": "#00ffff",
        "shape/imagefilter/sel_symbol/alpha": 0.8,
        "shape/rectzoom/line/style": "SolidLine",
        "shape/rectzoom/line/color": "#bbbbbb",
        "shape/rectzoom/line/width": 2,
        # not used -- start
        "shape/rectzoom/sel_line/style": "SolidLine",
        "shape/rectzoom/sel_line/color": "green",
        "shape/rectzoom/sel_line/width": 2,
        # not used -- end
        "shape/rectzoom/fill/color": "yellow",
        "shape/rectzoom/fill/style": "SolidPattern",
        "shape/rectzoom/fill/alpha": 0.1,
        # not used -- start
        "shape/rectzoom/symbol/marker": "NoSymbol",
        "shape/rectzoom/symbol/size": 0,
        "shape/rectzoom/symbol/edgecolor": MAIN_FG_COLOR,
        "shape/rectzoom/symbol/facecolor": "yellow",
        "shape/rectzoom/symbol/alpha": 1.0,
        "shape/rectzoom/sel_symbol/marker": "Rect",
        "shape/rectzoom/sel_symbol/size": 5,
        "shape/rectzoom/sel_symbol/edgecolor": MAIN_FG_COLOR,
        "shape/rectzoom/sel_symbol/facecolor": "yellow",
        "shape/rectzoom/sel_symbol/alpha": 1.0,
        # not used -- end
        "shape/axes/border/line/style": "SolidLine",
        "shape/axes/border/line/color": "magenta",
        "shape/axes/border/line/width": 1,
        "shape/axes/border/sel_line/style": "SolidLine",
        "shape/axes/border/sel_line/color": "magenta",
        "shape/axes/border/sel_line/width": 2,
        "shape/axes/border/fill/style": "NoBrush",
        "shape/axes/border/fill/color": "magenta",
        "shape/axes/border/fill/alpha": 0.0,
        "shape/axes/border/sel_fill/style": "SolidPattern",
        "shape/axes/border/sel_fill/color": "magenta",
        "shape/axes/border/sel_fill/alpha": 0.3,
        "shape/axes/border/symbol/marker": "NoSymbol",
        "shape/axes/border/symbol/size": 0,
        "shape/axes/border/symbol/edgecolor": MAIN_FG_COLOR,
        "shape/axes/border/symbol/facecolor": "yellow",
        "shape/axes/border/symbol/alpha": 1.0,
        "shape/axes/border/sel_symbol/marker": "NoSymbol",
        "shape/axes/border/sel_symbol/size": 5,
        "shape/axes/border/sel_symbol/edgecolor": MAIN_FG_COLOR,
        "shape/axes/border/sel_symbol/facecolor": "yellow",
        "shape/axes/border/sel_symbol/alpha": 1.0,
        "shape/axes/arrow_size": 8,
        "shape/axes/arrow_angle": 30,
        "shape/axes/xarrow_pen/style": "SolidLine",
        "shape/axes/xarrow_pen/color": "red",
        "shape/axes/xarrow_pen/width": 1,
        "shape/axes/xarrow_brush/color": "red",
        "shape/axes/xarrow_brush/alpha": 0.2,
        "shape/axes/yarrow_pen/style": "SolidLine",
        "shape/axes/yarrow_pen/color": "green",
        "shape/axes/yarrow_pen/width": 1,
        "shape/axes/yarrow_brush/color": "green",
        "shape/axes/yarrow_brush/alpha": 0.2,
        "shape/image_stats/line/style": "DashLine",
        "shape/image_stats/line/color": "#ff88dc",
        "shape/image_stats/line/width": 1,
        "shape/image_stats/fill/style": "SolidPattern",
        "shape/image_stats/fill/color": MAIN_BG_COLOR,
        "shape/image_stats/fill/alpha": 0.1,
        "shape/image_stats/symbol/marker": "Rect",
        "shape/image_stats/symbol/size": 3,
        "shape/image_stats/symbol/edgecolor": "#ff55dc",
        "shape/image_stats/symbol/facecolor": "#ff88dc",
        "shape/image_stats/symbol/alpha": 0.6,
        "shape/image_stats/sel_line/style": "DashLine",
        "shape/image_stats/sel_line/color": "#ff00dc",
        "shape/image_stats/sel_line/width": 1,
        "shape/image_stats/sel_fill/style": "SolidPattern",
        "shape/image_stats/sel_fill/color": MAIN_BG_COLOR,
        "shape/image_stats/sel_fill/alpha": 0.1,
        "shape/image_stats/sel_symbol/marker": "Rect",
        "shape/image_stats/sel_symbol/size": 5,
        "shape/image_stats/sel_symbol/edgecolor": "#ff00dc",
        "shape/image_stats/sel_symbol/facecolor": "#ff88dc",
        "shape/image_stats/sel_symbol/alpha": 0.7,
        "shape/cross_section/line/style": "DotLine",
        "shape/cross_section/line/color": "#ff5555",
        "shape/cross_section/line/width": 1,
        "shape/cross_section/fill/style": "SolidPattern",
        "shape/cross_section/fill/color": MAIN_BG_COLOR,
        "shape/cross_section/fill/alpha": 0.1,
        "shape/cross_section/symbol/marker": "Cross",
        "shape/cross_section/symbol/size": 100,
        "shape/cross_section/symbol/edgecolor": "cyan",
        "shape/cross_section/symbol/facecolor": "cyan",
        "shape/cross_section/symbol/alpha": 0.6,
        "shape/cross_section/sel_line/style": "DotLine",
        "shape/cross_section/sel_line/color": "#ff0000",
        "shape/cross_section/sel_line/width": 1,
        "shape/cross_section/sel_fill/style": "SolidPattern",
        "shape/cross_section/sel_fill/color": MAIN_BG_COLOR,
        "shape/cross_section/sel_fill/alpha": 0.1,
        "shape/cross_section/sel_symbol/marker": "Cross",
        "shape/cross_section/sel_symbol/size": 10000,
        "shape/cross_section/sel_symbol/edgecolor": "cyan",
        "shape/cross_section/sel_symbol/facecolor": "cyan",
        "shape/cross_section/sel_symbol/alpha": 0.7,
        "shape/average_cross_section/line/style": "DotLine",
        "shape/average_cross_section/line/color": "#ff5555",
        "shape/average_cross_section/line/width": 1,
        "shape/average_cross_section/fill/style": "SolidPattern",
        "shape/average_cross_section/fill/color": MAIN_BG_COLOR,
        "shape/average_cross_section/fill/alpha": 0.1,
        "shape/average_cross_section/symbol/marker": "Diamond",
        "shape/average_cross_section/symbol/size": 7,
        "shape/average_cross_section/symbol/edgecolor": "#ff5555",
        "shape/average_cross_section/symbol/facecolor": "#ff5555",
        "shape/average_cross_section/symbol/alpha": 0.6,
        "shape/average_cross_section/sel_line/style": "DotLine",
        "shape/average_cross_section/sel_line/color": "#ff0000",
        "shape/average_cross_section/sel_line/width": 1,
        "shape/average_cross_section/sel_fill/style": "SolidPattern",
        "shape/average_cross_section/sel_fill/color": MAIN_BG_COLOR,
        "shape/average_cross_section/sel_fill/alpha": 0.1,
        "shape/average_cross_section/sel_symbol/marker": "Diamond",
        "shape/average_cross_section/sel_symbol/size": 9,
        "shape/average_cross_section/sel_symbol/edgecolor": "#aa0000",
        "shape/average_cross_section/sel_symbol/facecolor": "#ff0000",
        "shape/average_cross_section/sel_symbol/alpha": 0.7,
        "shape/mask/line/style": "DotLine",
        "shape/mask/line/color": "#5555ff",
        "shape/mask/line/width": 1,
        "shape/mask/fill/style": "SolidPattern",
        "shape/mask/fill/color": MAIN_BG_COLOR,
        "shape/mask/fill/alpha": 0.1,
        "shape/mask/symbol/marker": "Rect",
        "shape/mask/symbol/size": 7,
        "shape/mask/symbol/edgecolor": "#5555ff",
        "shape/mask/symbol/facecolor": "#5555ff",
        "shape/mask/symbol/alpha": 0.6,
        "shape/mask/sel_line/style": "DotLine",
        "shape/mask/sel_line/color": "#0000ff",
        "shape/mask/sel_line/width": 1,
        "shape/mask/sel_fill/style": "SolidPattern",
        "shape/mask/sel_fill/color": MAIN_BG_COLOR,
        "shape/mask/sel_fill/alpha": 0.1,
        "shape/mask/sel_symbol/marker": "Rect",
        "shape/mask/sel_symbol/size": 9,
        "shape/mask/sel_symbol/edgecolor": "#0000aa",
        "shape/mask/sel_symbol/facecolor": "#0000ff",
        "shape/mask/sel_symbol/alpha": 0.7,
        "shape/point/line/style": "SolidLine",
        "shape/point/line/color": "#ffff00",
        "shape/point/line/width": 1,
        "shape/point/sel_line/style": "SolidLine",
        "shape/point/sel_line/color": "#00ff00",
        "shape/point/sel_line/width": 1,
        "shape/point/fill/style": "NoBrush",
        "shape/point/sel_fill/style": "NoBrush",
        "shape/point/symbol/marker": "XCross",
        "shape/point/symbol/size": 9,
        "shape/point/symbol/edgecolor": "#ffff00",
        "shape/point/symbol/facecolor": "#ffff00",
        "shape/point/symbol/alpha": 1.0,
        "shape/point/sel_symbol/marker": "XCross",
        "shape/point/sel_symbol/size": 12,
        "shape/point/sel_symbol/edgecolor": "#00aa00",
        "shape/point/sel_symbol/facecolor": "#00ff00",
        "shape/point/sel_symbol/alpha": 0.7,
        "shape/segment/line/style": "SolidLine",
        "shape/segment/line/color": "#ffff00",
        "shape/segment/line/width": 1,
        "shape/segment/sel_line/style": "SolidLine",
        "shape/segment/sel_line/color": "#00ff00",
        "shape/segment/sel_line/width": 1,
        "shape/segment/fill/style": "NoBrush",
        "shape/segment/sel_fill/style": "NoBrush",
        "shape/segment/symbol/marker": "XCross",
        "shape/segment/symbol/size": 9,
        "shape/segment/symbol/edgecolor": "#ffff00",
        "shape/segment/symbol/facecolor": "#ffff00",
        "shape/segment/symbol/alpha": 1.0,
        "shape/segment/sel_symbol/marker": "XCross",
        "shape/segment/sel_symbol/size": 12,
        "shape/segment/sel_symbol/edgecolor": "#00aa00",
        "shape/segment/sel_symbol/facecolor": "#00ff00",
        "shape/segment/sel_symbol/alpha": 0.7,
        "shape/label/symbol/marker": "NoSymbol",
        "shape/label/symbol/size": 0,
        "shape/label/symbol/edgecolor": MAIN_BG_COLOR,
        "shape/label/symbol/facecolor": MAIN_BG_COLOR,
        "shape/label/border/style": "SolidLine",
        "shape/label/border/color": "#cbcbcb",
        "shape/label/border/width": 0,
        "shape/label/font/size": 8,
        "shape/label/font/family": "default",
        "shape/label/font/bold": False,
        "shape/label/font/italic": False,
        "shape/label/color": MAIN_BG_COLOR,
        "shape/label/bgcolor": MAIN_FG_COLOR,
        "shape/label/bgalpha": 0.25,
        "shape/label/abspos": False,
        "shape/label/move_anchor": True,
        "label/symbol/marker": "NoSymbol",
        "label/symbol/size": 0,
        "label/symbol/edgecolor": MAIN_BG_COLOR,
        "label/symbol/facecolor": MAIN_BG_COLOR,
        "label/border/style": "SolidLine",
        "label/border/color": "#cbcbcb",
        "label/border/width": 1,
        "label/font/size": 9,
        "label/font/family": "default",
        "label/font/bold": False,
        "label/font/italic": False,
        "label/color": MAIN_FG_COLOR,
        "label/bgcolor": MAIN_BG_COLOR,
        "label/bgalpha": 0.8,
        "label/anchor": "TL",
        "label/xc": 0,
        "label/yc": 0,
        "label/abspos": False,
        "label/absg": "TR",
        "label/xg": 0.0,
        "label/yg": 0.0,
        # info_label: used in builder.make.computation for example
        "info_label/symbol/marker": "NoSymbol",
        "info_label/symbol/size": 0,
        "info_label/symbol/edgecolor": MAIN_BG_COLOR,
        "info_label/symbol/facecolor": MAIN_BG_COLOR,
        "info_label/border/style": "SolidLine",
        "info_label/border/color": "#cbcbcb",
        "info_label/border/width": 1,
        "info_label/font/size": 9,
        "info_label/font/family": "default",
        "info_label/font/bold": False,
        "info_label/font/italic": False,
        "info_label/color": MAIN_FG_COLOR,
        "info_label/bgcolor": MAIN_BG_COLOR,
        "info_label/bgalpha": 0.8,
        "info_label/anchor": "TL",
        "info_label/xc": 0,
        "info_label/yc": 0,
        "info_label/abspos": True,
        "info_label/absg": "TR",
        "info_label/xg": 0.0,
        "info_label/yg": 0.0,
        "legend/border/style": "SolidLine",
        "legend/border/color": "#cbcbcb",
        "legend/border/width": 1,
        "legend/font/size": 8,
        "legend/font/family": "default",
        "legend/font/bold": False,
        "legend/font/italic": False,
        "legend/color": MAIN_FG_COLOR,
        "legend/bgcolor": MAIN_BG_COLOR,
        "legend/bgalpha": 0.8,
        "legend/anchor": "TR",
        "legend/xc": 0,
        "legend/yc": 0,
        "legend/abspos": True,
        "legend/absg": "TR",
        "legend/xg": 0.0,
        "legend/yg": 0.0,
    },
    "histogram": {
        "antialiasing": False,
        "title/font/size": 11,
        "title/font/bold": False,
        "label/font/size": 9,
        "label/font/bold": False,
        "curve/line/style": "NoPen",
        "curve/line/color": "green",
        "curve/line/width": 1,
        "curve/symbol/marker": "NoSymbol",
        "curve/symbol/size": 0,
        "curve/symbol/edgecolor": MAIN_BG_COLOR,
        "curve/symbol/facecolor": MAIN_FG_COLOR,
        "curve/symbol/alpha": 1.0,
        "curve/shade": 0.85,
        "curve/curvestyle": "Steps",
        "curve/label": "",
        "range/line/style": "SolidLine",
        "range/line/color": "#ff9393",
        "range/line/width": 1,
        "range/sel_line/style": "SolidLine",
        "range/sel_line/color": "red",
        "range/sel_line/width": 2,
        "range/fill": "red",
        "range/shade": 0.15,
        "range/symbol/marker": "Ellipse",
        "range/symbol/size": 7,
        "range/symbol/edgecolor": MAIN_BG_COLOR,
        "range/symbol/facecolor": "#ff9393",
        "range/symbol/alpha": 1.0,
        "range/sel_symbol/marker": "Ellipse",
        "range/sel_symbol/size": 9,
        "range/sel_symbol/edgecolor": MAIN_BG_COLOR,
        "range/sel_symbol/facecolor": "red",
        "range/sel_symbol/alpha": 0.9,
        "range/multi/color": "#806060",
        # Default parameters for plot axes
        "axis/title": "",
        "axis/color": MAIN_FG_COLOR,
        "axis/title_font/size": 7,
        "axis/title_font/family": "default",
        "axis/title_font/bold": False,
        "axis/title_font/italic": False,
        "axis/ticks_font/size": 7,
        "axis/ticks_font/family": "default",
        "axis/ticks_font/bold": False,
        "axis/ticks_font/italic": False,
        "grid/background": MAIN_BG_COLOR,
        "grid/maj_xenabled": True,
        "grid/maj_yenabled": True,
        "grid/maj_line/color": "darkgray",
        "grid/maj_line/width": 1,
        "grid/maj_line/style": "DotLine",
        "grid/min_xenabled": True,
        "grid/min_yenabled": False,
        "grid/min_line/color": "#eaeaea",
        "grid/min_line/width": 1,
        "grid/min_line/style": "DotLine",
        "marker/curve/symbol/marker": "Ellipse",
        "marker/curve/symbol/edgecolor": MAIN_FG_COLOR,
        "marker/curve/symbol/facecolor": "blue",
        "marker/curve/symbol/alpha": 0.8,
        "marker/curve/symbol/size": 8,
        "marker/curve/text/font/size": 8,
        "marker/curve/text/font/family": "default",
        "marker/curve/text/font/bold": False,
        "marker/curve/text/font/italic": False,
        "marker/curve/text/textcolor": "darkBlue",
        "marker/curve/text/background_color": "blue",
        "marker/curve/text/background_alpha": 0.2,
        "marker/curve/pen/style": "SolidLine",
        "marker/curve/pen/color": MAIN_FG_COLOR,
        "marker/curve/pen/width": 0,
        "marker/curve/markerstyle": "NoLine",
        "marker/cross/symbol/marker": "Cross",
        "marker/cross/symbol/edgecolor": MAIN_FG_COLOR,
        "marker/cross/symbol/facecolor": "red",
        "marker/cross/symbol/alpha": 1.0,
        "marker/cross/symbol/size": 8,
        "marker/cross/text/font/family": "default",
        "marker/cross/text/font/size": 8,
        "marker/cross/text/font/bold": False,
        "marker/cross/text/font/italic": False,
        "marker/cross/text/textcolor": "darkBlue",
        "marker/cross/text/background_color": "blue",
        "marker/cross/text/background_alpha": 0.2,
        "marker/cross/pen/style": "SolidLine",
        "marker/cross/pen/color": MAIN_FG_COLOR,
        "marker/cross/pen/width": 1,
        "marker/cross/markerstyle": "Cross",
    },
    "cross_section": {
        "antialiasing": False,
        "title/font/size": 11,
        "title/font/bold": False,
        "label/font/size": 9,
        "label/font/bold": False,
        "curve/line/style": "SolidLine",
        "curve/line/color": "blue",
        "curve/line/width": 1,
        "curve/symbol/marker": "NoSymbol",
        "curve/symbol/size": 0,
        "curve/symbol/edgecolor": MAIN_BG_COLOR,
        "curve/symbol/facecolor": MAIN_FG_COLOR,
        "curve/symbol/alpha": 1.0,
        "curve/shade": 0.0,
        "curve/label": "",
        "range/line/style": "SolidLine",
        "range/line/color": "#ff9393",
        "range/line/width": 1,
        "range/sel_line/style": "SolidLine",
        "range/sel_line/color": "red",
        "range/sel_line/width": 2,
        "range/fill": "red",
        "range/shade": 0.15,
        "range/symbol/marker": "NoSymbol",
        "range/symbol/size": 5,
        "range/symbol/edgecolor": MAIN_FG_COLOR,
        "range/symbol/facecolor": "yellow",
        "range/symbol/alpha": 1.0,
        "range/sel_symbol/marker": "Ellipse",
        "range/sel_symbol/size": 9,
        "range/sel_symbol/edgecolor": MAIN_BG_COLOR,
        "range/sel_symbol/facecolor": "red",
        "range/sel_symbol/alpha": 0.9,
        "range/multi/color": "#806060",
        # Default parameters for plot axes
        "axis/title": "",
        "axis/color": MAIN_FG_COLOR,
        "axis/title_font/size": 7,
        "axis/title_font/family": "default",
        "axis/title_font/bold": False,
        "axis/title_font/italic": False,
        "axis/ticks_font/size": 7,
        "axis/ticks_font/family": "default",
        "axis/ticks_font/bold": False,
        "axis/ticks_font/italic": False,
        "grid/background": MAIN_BG_COLOR,
        "grid/maj_xenabled": True,
        "grid/maj_yenabled": True,
        "grid/maj_line/color": "darkgray",
        "grid/maj_line/width": 1,
        "grid/maj_line/style": "DotLine",
        "grid/min_xenabled": False,
        "grid/min_yenabled": False,
        "grid/min_line/color": "#eaeaea",
        "grid/min_line/width": 1,
        "grid/min_line/style": "DotLine",
        "marker/curve/symbol/marker": "Ellipse",
        "marker/curve/symbol/edgecolor": MAIN_FG_COLOR,
        "marker/curve/symbol/facecolor": "blue",
        "marker/curve/symbol/alpha": 0.8,
        "marker/curve/symbol/size": 8,
        "marker/curve/text/font/size": 8,
        "marker/curve/text/font/family": "default",
        "marker/curve/text/font/bold": False,
        "marker/curve/text/font/italic": False,
        "marker/curve/text/textcolor": "darkBlue",
        "marker/curve/text/background_color": "blue",
        "marker/curve/text/background_alpha": 0.2,
        "marker/curve/pen/style": "SolidLine",
        "marker/curve/pen/color": MAIN_FG_COLOR,
        "marker/curve/pen/width": 0,
        "marker/curve/markerstyle": "NoLine",
        "marker/cross/symbol/marker": "Cross",
        "marker/cross/symbol/edgecolor": MAIN_FG_COLOR,
        "marker/cross/symbol/facecolor": "red",
        "marker/cross/symbol/alpha": 1.0,
        "marker/cross/symbol/size": 8,
        "marker/cross/text/font/family": "default",
        "marker/cross/text/font/size": 8,
        "marker/cross/text/font/bold": False,
        "marker/cross/text/font/italic": False,
        "marker/cross/text/textcolor": "darkBlue",
        "marker/cross/text/background_color": "blue",
        "marker/cross/text/background_alpha": 0.2,
        "marker/cross/pen/style": "SolidLine",
        "marker/cross/pen/color": MAIN_FG_COLOR,
        "marker/cross/pen/width": 1,
        "marker/cross/markerstyle": "Cross",
    },
}

from guidata.config import CONF

CONF.update_defaults(DEFAULTS)



# -*- coding: utf-8 -*-
#
# Copyright © 2009-2012 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guiqwt/__init__.py for details)

"""Load/save items using Python's pickle protocol"""


SHOW = True  # Show test in GUI-based test launcher

# WARNING:
# This script requires read/write permissions on current directory

from qtpy.QtGui import QFont

import os
import os.path as osp
import numpy as np

from guiqwt.plot import ImageDialog
from guiqwt.builder import make
from guiqwt.shapes import PolygonShape, Axes
from guiqwt.tools import LoadItemsTool, SaveItemsTool, ImageMaskTool


def build_items():
    x = np.linspace(-10, 10, 200)
    y = np.sin(np.sin(np.sin(x)))
    filename = osp.join(osp.dirname(__file__), "brain.png")
    items = [
        make.curve(x, y, color="b"),
        make.image(filename=filename),
        make.trimage(filename=filename),
        make.maskedimage(
            filename=filename,
            colormap="gray",
            show_mask=True,
            xdata=[0, 40],
            ydata=[0, 50],
        ),
        make.label("Relative position <b>outside</b>", (x[0], y[0]), (-10, -10), "BR"),
        make.label("Relative position <i>inside</i>", (x[0], y[0]), (10, 10), "TL"),
        make.label("Absolute position", "R", (0, 0), "R"),
        make.legend("TR"),
        make.rectangle(-3, -0.8, -0.5, -1.0, "rc1"),
        make.segment(-3, -0.8, -0.5, -1.0, "se1"),
        make.ellipse(-10, 0.0, 0, 0, "el1"),
        make.annotated_rectangle(0.5, 0.8, 3, 1.0, "rc1", "tutu"),
        make.annotated_segment(-1, -1, 1, 1.0, "rc1", "tutu"),
        Axes((0, 0), (1, 0), (0, 1)),
        PolygonShape(
            np.array(
                [
                    [150.0, 330.0],
                    [270.0, 520.0],
                    [470.0, 480.0],
                    [520.0, 360.0],
                    [460.0, 200.0],
                    [250.0, 240.0],
                ]
            )
        ),
    ]
    return items


class IOTest(object):
    FNAME = None

    def __init__(self):
        self.dlg = None

    @property
    def plot(self):
        if self.dlg is not None:
            return self.dlg.get_plot()

    def run(self):
        """Run test"""
        self.create_dialog()
        self.add_items()
        self.dlg.exec_()
        print("Saving items...", end=" ")
        self.save_items()
        print("OK")

    def create_dialog(self):
        self.dlg = dlg = ImageDialog(
            edit=False,
            toolbar=True,
            wintitle="Load/save test",
            options=dict(title="Title", xlabel="xlabel", ylabel="ylabel"),
        )
        dlg.add_separator_tool()
        dlg.add_tool(LoadItemsTool)
        dlg.add_tool(SaveItemsTool)
        dlg.add_tool(ImageMaskTool)

    def add_items(self):
        if os.access(self.FNAME, os.R_OK):
            print("Restoring items...", end=" ")
            self.restore_items()
            print("OK")
        else:
            for item in build_items():
                self.plot.add_item(item)
            print("Building items and add them to plotting canvas", end=" ")
        self.plot.set_axis_font("left", QFont("Courier"))
        self.dlg.get_itemlist_panel().show()
        self.plot.set_items_readonly(False)

    def restore_items(self):
        raise NotImplementedError

    def save_items(self):
        raise NotImplementedError


class PickleTest(IOTest):
    FNAME = "loadsavecanvas.pickle"

    def restore_items(self):
        f = open(self.FNAME, "rb")
        self.plot.restore_items(f)

    def save_items(self):
        f = open(self.FNAME, "wb")
        self.plot.save_items(f)


if __name__ == "__main__":
    import guidata

    _app = guidata.qapplication()
    test = PickleTest()
    test.run()



# -*- coding: utf-8 -*-
#
# Copyright © 2010 Ludovic Aubry
# Copyright © 2022 Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guidata/__init__.py for details)

"""SyncPlotDialog test"""

from guiqwt.config import _
from guidata.configtools import get_icon

from qtpy import QtWidgets as QW
from qtpy import QtGui as QG

from guiqwt.baseplot import BasePlot
from guiqwt.plot import SubplotWidget, PlotManager
from guiqwt.builder import make
from guiqwt.curve import CurvePlot

SHOW = False  # Show test in GUI-based test launcher


class SyncPlotDialog(QW.QDialog):
    """Dialog demonstrating plot synchronization feature"""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle(self.__doc__)
        self.setWindowIcon(get_icon("guiqwt.svg"))

        self.manager = manager = PlotManager(None)
        manager.set_main(self)
        self.subplotwidget = spwidget = SubplotWidget(manager, parent=self)
        self.setLayout(QW.QVBoxLayout())
        toolbar = QW.QToolBar(_("Tools"))
        manager.add_toolbar(toolbar)
        self.layout().addWidget(toolbar)
        self.layout().addWidget(spwidget)

        plot1 = CurvePlot(title="TL")
        plot2 = CurvePlot(title="TR")
        plot3 = CurvePlot(title="BL")
        plot4 = CurvePlot(title="BR")
        spwidget.add_subplot(plot1, 0, 0, "1")
        spwidget.add_subplot(plot2, 0, 1, "2")
        spwidget.add_subplot(plot3, 1, 0, "3")
        spwidget.add_subplot(plot4, 1, 1, "4")
        spwidget.add_itemlist()
        manager.synchronize_axis(BasePlot.X_BOTTOM, ["1", "3"])
        manager.synchronize_axis(BasePlot.X_BOTTOM, ["2", "4"])
        manager.synchronize_axis(BasePlot.Y_LEFT, ["1", "2"])
        manager.synchronize_axis(BasePlot.Y_LEFT, ["3", "4"])

        self.manager.register_all_curve_tools()


def plot(items1, items2, items3, items4):
    """Plot items in SyncPlotDialog"""
    dlg = SyncPlotDialog()
    items = [items1, items2, items3, items4]
    for i, plot in enumerate(dlg.subplotwidget.plots):
        for item in items[i]:
            plot.add_item(item)
        plot.set_axis_font("left", QG.QFont("Courier"))
        plot.set_items_readonly(False)
    dlg.manager.get_panel("itemlist").show()
    dlg.show()
    dlg.exec_()


def test():
    """Test"""
    # -- Create QApplication
    import guidata

    _app = guidata.qapplication()
    # --
    from numpy import linspace, sin

    x = linspace(-10, 10, 200)
    dy = x / 100.0
    y = sin(sin(sin(x)))
    x2 = linspace(-10, 10, 20)
    y2 = sin(sin(sin(x2)))
    plot(
        [
            make.curve(x, y, color="b"),
            make.label(
                "Relative position <b>outside</b>", (x[0], y[0]), (-10, -10), "BR"
            ),
        ],
        [
            make.curve(x2, y2, color="g"),
        ],
        [
            make.curve(x, sin(2 * y), color="r"),
            make.label("Relative position <i>inside</i>", (x[0], y[0]), (10, 10), "TL"),
        ],
        [
            make.merror(x, y / 2, dy),
            make.label("Absolute position", "R", (0, 0), "R"),
            make.legend("TR"),
        ],
    )


if __name__ == "__main__":
    test()
